
svgOpen <- function(filename="Rplots.svg", width=200, height=200) {
  # For viewing using Adobe SVG Viewer in IE
  # create a "wrapper" html file
  htmlfile <- file(paste(filename, ".html", sep=""), "w")
  cat(paste('<object data="', filename, '" type="image/svg-xml"',
            'width="', width, '" height="', height, '"> </object>\n',
            sep=''), file=htmlfile)
  
  svgdev <- svgDevice(file(filename, 'w'), width, height)
  svgHeader(svgdev)
  return(svgdev)
}

svgClose <- function(svgdev) {
  svgFooter(svgdev)
  close(svgDevFile(svgdev))
}

# The locations and dimensions MUST be in user coordinates
svgPushViewport <- function(x=0, y=0, width=1, height=1,
                            xscale=0:1, yscale=0:1,
                            style=svgStyle(), svgdev=svgDevice()) {
  pushsvgTransform(x, y, width, height, xscale, yscale, svgdev)
  incindent(svgdev)
  catsvg(paste('<g ',
               svgStyleCSS(style), 
               '>\n',
               sep=""), svgdev)
}

svgPopViewport <- function(n=1, svgdev=svgDevice()) {
  for (i in 1:n) {
    popsvgTransform(svgdev)
    catsvg('</g>\n', svgdev)
    decindent(svgdev)
  }
}

svgStartGroup <- function(style=svgStyle(), svgdev=svgDevice()) {
  incindent(svgdev)
  catsvg(paste('<g ',
               'id="', svgID(svgdev), '" ',
               svgStyleCSS(style), 
               '>\n',
               sep=""), svgdev)
  incID(svgdev)
}

svgEndGroup <- function(svgdev=svgDevice()) {
  catsvg('</g>\n', svgdev)
  decindent(svgdev)
}

svgStartLink <- function(href="", svgdev=svgDevice()) {
  incindent(svgdev)
  catsvg(paste('<a xlink:href="', href, '">\n',
               sep=""), svgdev)  
}

svgEndLink <- function(svgdev=svgDevice()) {
  catsvg('</a>\n', svgdev)
  decindent(svgdev)
}

svgAnimate <- function(attrib, values, id, duration, rep, revert,
                       svgdev=svgDevice()) {
  n <- if (is.null(id)) 1 else length(unique(id))
  catsvg(paste('<animate ',
               'xlink:href="#', svgID(svgdev) + (1:n - 1), '" ',
               'attributeName="', attrib, '" ',
               'dur="', duration, 's" ',
               'values="', values, '" ',
               'repeatCount="',
               if (is.numeric(rep)) rep else if (rep) "indefinite" else 1,
               '" ',
               'fill="',
               if (revert) "remove" else "freeze",
               '" ',               
               '/>\n', sep=""),
         svgdev)
}

# This and svgAnimateY are untested with id != NULL
# and I have a strong suspicion there may be problems
# because tapply returns a list -- see svgAnimatePoints
# for ideas for a possible solution (esp. the lpaste function)
svgAnimateX <- function(attrib, values, id, duration, rep, revert,
                        svgdev=svgDevice()) {
  svgAnimate(attrib,
             if (is.null(id))
               paste(svgtx(values, svgdev), collapse=';')
             else
               tapply(svgtx(values, svgdev), id, paste, collapse=';'),
             id, duration, rep, revert, svgdev)  
}

svgAnimateY <- function(attrib, values, id, duration, rep, revert,
                        svgdev=svgDevice()) {
  svgAnimate(attrib,
             if (is.null(id))
               paste(svgty(values, svgdev), collapse=';')
             else
               tapply(svgty(values, svgdev), id, paste, collapse=';'),
             id, duration, rep, revert, svgdev)  
}

svgAnimateWidth <- function(attrib, values, id, duration, rep, revert,
                            svgdev=svgDevice()) {
  svgAnimate(attrib,
             if (is.null(id))
               paste(svgtw(values, svgdev), collapse=';')
             else
               tapply(svgtw(values, svgdev), id, paste, collapse=';'),
             id, duration, rep, revert, svgdev)  
}

svgAnimateHeight <- function(attrib, values, id, duration, rep, revert,
                             svgdev=svgDevice()) {
  svgAnimate(attrib,
             if (is.null(id))
               paste(svgth(values, svgdev), collapse=';')
             else
               tapply(svgth(values, svgdev), id, paste, collapse=';'),
             id, duration, rep, revert, svgdev)  
}

svgAnimateDim <- function(attrib, values, id, duration, rep, revert,
                            svgdev=svgDevice()) {
  svgAnimate(attrib,
             if (is.null(id))
               paste(svgtd(values, svgdev), collapse=';')
             else
               tapply(svgtd(values, svgdev), id, paste, collapse=';'),
             id, duration, rep, revert, svgdev)  
}

# DON'T call this with a list of length < 2!
old.lpaste <- function(alist, collapse) {
  n <- length(alist)
  if (n == 2)
    result <- paste(alist[[1]], alist[[2]])
  else 
    result <- paste(alist[[n]], lpaste(alist[1:(n-1)], collapse))
  paste(result, collapse=collapse)
}

lpaste <- function(alist, collapse) {
  n <- length(alist)
  result <- alist[[1]]
  for (i in 2:n)
    result <- paste(result, alist[[i]])
  paste(result, collapse=collapse)
}

svgAnimatePoints <- function(xvalues, yvalues, id, duration, rep, revert,
                             svgdev=svgDevice()) {
  if (is.null(id))
    warning("Only one point to animate")
  else
    svgAnimate("points",
               # The tapply splits the points into a list
               # of successive x, y sets corresponding to the
               # animation values for each point on the line
               lpaste(tapply(paste(svgtx(xvalues, svgdev),
                                   svgty(yvalues, svgdev), sep=","),
                             id, paste), collapse=";"),
               NULL, duration, rep, revert, svgdev)  
}

svgAnimateTransform <- function(attrib, values, id, duration, rep, revert,
                                svgdev=svgDevice()) {
  n <- if (is.null(id)) 1 else length(unique(id))
  catsvg(paste('<animateTransform ',
               'xlink:href="#', svgID(svgdev) + (1:n - 1), '" ',
               'attributeName="transform" ',
               'type="', attrib, '" ',
               'dur="', duration, 's" ',
               'values="', values, '" ',
               'repeatCount="',
               if (is.numeric(rep)) rep else if (rep) "indefinite" else 1,
               '" ',
               'fill="',
               if (revert) "remove" else "freeze",
               '" ',
               '/>\n', sep=""),
         svgdev)
}

svgAnimateTranslation <- function(xvalues, yvalues, id, duration, rep, revert,
                                  svgdev=svgDevice()) {
  svgAnimateTransform("translate",
                      if (is.null(id))
                        paste(svgtx(xvalues, svgdev),
                                   svgty(yvalues, svgdev),
                              sep=",", collapse=';')
                      else
                        tapply(paste(svgtx(xvalues, svgdev),
                                     svgty(yvalues, svgdev), sep=","),
                               id, paste, collapse=';'),
                      id, duration, rep, revert, svgdev)  
}

svgLines <- function(x, y, style=svgStyle(), svgdev=svgDevice()) {
  n <- max(length(x), length(y))
  catsvg(paste('<polyline ',
               'id="', svgID(svgdev), '" ',
               'points="',
               paste(rep(svgtx(x, svgdev), length=n), ",",
                     rep(svgty(y, svgdev), length=n), sep="",
                     collapse=" "),
               '" ', 
               svgStyleCSS(style), 
               ' />\n', sep=""),
         svgdev)  
  incID(svgdev)
}

svgPolygon <- function(x, y, style=svgStyle(), svgdev=svgDevice()) {
  if (length(x) != length(y))
    stop("x and y must be same length")
  n <- length(x)
  catsvg(paste('<polygon ',
               'id="', svgID(svgdev), '" ',
               'points="', 
               paste(rep(svgtx(x, svgdev), length=n), ",",
                     rep(svgty(y, svgdev), length=n), sep="",
                     collapse=" "),
               '" ', 
               svgStyleCSS(style), 
               ' />\n', sep=""),
         svgdev);  
  incID(svgdev)
}

svgRect <- function(x, y, width, height,
                    style=svgStyle(), svgdev=svgDevice()) {
  n <- max(length(x), length(y), length(width), length(height))
  rects <- paste('<rect ',
                 'id="', svgID(svgdev) + (1:n - 1), '" ',
                 'x="', rep(svgtx(x, svgdev), length=n), '" ',
                 'y="', rep(svgty(y, svgdev), length=n), '" ',
                 'width="', rep(svgtw(width, svgdev), length=n), '" ',
                 'height="', rep(svgth(height, svgdev), length=n), '" ',
                 svgStyleCSS(style),
                 ' />\n',
                 sep="")
  catsvg(rects, svgdev)
  incID(svgdev, n)
}

svgText <- function(x, y, text, hjust="left", vjust="bottom", rot=0,
                    style=svgStyle(), svgdev=svgDevice()) {
  n <- max(length(x), length(y), length(text))
  # Flip the y-direction again so that text is drawn "upright"
  # Do the flip in a separate <g> so that can animate the
  # translation easily
  # Use a tspan to do the vertical alignment
  texts <- paste('<g ',
                 'id="', svgID(svgdev) + (1:n - 1), '" ',
                 # Only draw a REALLY thin line for the text outline
                 'stroke-width=".1" ',
                 'transform="translate(',
                 rep(svgtx(x, svgdev), length=n), ', ',
                 rep(svgty(y, svgdev), length=n), ') ',
                 '">\n',
                 '<g transform="scale(1, -1)">\n',
                 '<text x="0" y="0" ',
                 if (rot != 0) {
                   paste('transform="rotate(',
                         rep(rot, length=n),
                         ')" ', sep="")
                 } else "",
                 textAnchor(hjust), ' ',
                 svgStyleCSS(style),
                 ' >\n',
                 '<tspan ',
                 baselineShift(vjust), '>',
                 rep(text, length=n),
                 '</tspan>\n',
                 '</text>\n',
                 '</g>\n',
                 '</g>\n',
                 sep="")
  incindent(svgdev)
  catsvg(texts, svgdev)
  decindent(svgdev)
  incID(svgdev, n)
}

svgCircle <- function(x, y, r,
                      style=svgStyle(), svgdev=svgDevice()) {
  n <- max(length(x), length(y), length(r))
  circles <- paste('<circle ',
                 'id="', svgID(svgdev) + (1:n - 1), '" ',
                 'cx="', rep(svgtx(x, svgdev), length=n), '" ',
                 'cy="', rep(svgty(y, svgdev), length=n), '" ',
                 'r="', rep(svgtd(r, svgdev), length=n), '" ',
                 svgStyleCSS(style),
                 ' />\n',
                 sep="")
  catsvg(circles, svgdev)
  incID(svgdev, n)
}

#############
# Internal functions
#############

# SVG Devices
# A device is an environment so that we can modify values
# stored within it.
# Store a list of transformation functions for
# x, y, width, and height;  this will allow viewports
# to be defined within user coordinates (see svgPushViewport
# and svgPopViewport)
svgDevice <- function(file="", width=200, height=200) {
  dev <- new.env(FALSE, NULL)
  assign("file", file, env=dev)
  assign("width", width, env=dev)
  assign("height", height, env=dev)
  assign("tx", list(), env=dev)
  assign("ty", list(), env=dev)
  assign("tw", list(), env=dev)
  assign("th", list(), env=dev)
  assign("indent", "", env=dev)
  assign("id", 1, env=dev)
  return(dev)
}

svgDevFile <- function(svgdev) {
  get("file", env=svgdev)
}

svgDevWidth <- function(svgdev) {
  get("width", env=svgdev)
}

svgDevHeight <- function(svgdev) {
  get("height", env=svgdev)
}

svgDevtx <- function(svgdev) {
  get("tx", env=svgdev)
}

svgDevty <- function(svgdev) {
  get("ty", env=svgdev)
}

svgDevtw <- function(svgdev) {
  get("tw", env=svgdev)
}

svgDevth <- function(svgdev) {
  get("th", env=svgdev)
}

pushsvgDevtx <- function(tx, svgdev) {
  assign("tx", c(get("tx", env=svgdev), tx), env=svgdev)
}

popsvgDevtx <- function(svgdev) {
  prevtx <- get("tx", env=svgdev)
  assign("tx", prevtx[1:(length(prevtx)-1)], env=svgdev)
}

pushsvgDevty <- function(ty, svgdev) {
  assign("ty", c(get("ty", env=svgdev), ty), env=svgdev)
}

popsvgDevty <- function(svgdev) {
  prevty <- get("ty", env=svgdev)
  assign("ty", prevty[1:(length(prevty)-1)], env=svgdev)
}

pushsvgDevtw <- function(tw, svgdev) {
  assign("tw", c(get("tw", env=svgdev), tw), env=svgdev)
}

popsvgDevtw <- function(svgdev) {
  prevtw <- get("tw", env=svgdev)
  assign("tw", prevtw[1:(length(prevtw)-1)], env=svgdev)
}

pushsvgDevth <- function(th, svgdev) {
  assign("th", c(get("th", env=svgdev), th), env=svgdev)
}

popsvgDevth <- function(svgdev) {
  prevth <- get("th", env=svgdev)
  assign("th", prevth[1:(length(prevth)-1)], env=svgdev)
}

svgID <- function(svgdev) {
  get("id", env=svgdev)
}

# SVG output
catsvg <- function(text, svgdev) {
  cat(paste(get("indent", env=svgdev), text, sep=""),
      file=svgDevFile(svgdev))
}

decindent <- function(svgdev) {
  indent <- get("indent", env=svgdev)
  assign("indent", substr(indent, 1, nchar(indent) - 2),
         env=svgdev)
}

incindent <- function(svgdev) {
  assign("indent", paste(get("indent", env=svgdev), "  ", sep=""),
         env=svgdev)
}

incID <- function(svgdev, n=1) {
  assign("id", get("id", env=svgdev) + n, env=svgdev)
}

svgHeader <- function(svgdev=svgDevice()) {
  catsvg('<?xml version="1.0" standalone="no"?>\n', svgdev)
  catsvg('<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 20010904//EN" ', svgdev)
  catsvg('"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">\n',
         svgdev)
  catsvg('<svg>\n', svgdev)
  # Invert the y-axis so that y and height values measure "up"
  catsvg(paste('<g transform="translate(0, ',
               svgDevHeight(svgdev), ') ',
               ' scale(1, -1)">\n',
               sep=""), svgdev)
}

svgFooter <- function(svgdev=svgDevice()) {
  catsvg('</g>\n', svgdev);
  catsvg('</svg>\n', svgdev);
}

# SVG styling
svgStyle <- function(...) {
  list(...)
}

listToSVGStyle <- function(alist) {
  alist
}

emptyStyle <- function(svgstyle) {
  length(svgstyle) == 0
}

svgStyleCSS <- function(svgstyle) {
  if (emptyStyle(svgstyle)) {
    ""
  } else {
    paste('style="', paste(names(svgstyle), ":",
                           paste(svgstyle), sep="", collapse="; "),
          '"', sep="")
  }
}

# Specifying text justification
textAnchor <- function(hjust) {
  paste("text-anchor=",
        switch(hjust,
               left='"start"',
               center='"middle"',
               centre='"middle"',
               right='"end"',
               '"start"'),
        sep="")
}

dominantBaseline <- function(vjust) {
  paste("dominant-baseline=",
        switch(vjust,
               bottom='"auto"',
               center='"middle"',
               centre='"middle"',
               top='"text-top"',
               '"baseline"'),
        sep="")
}

baselineShift <- function(vjust) {
  paste('baseline-shift=',
        switch(vjust,
               bottom='"0%"',
               center='"-50%"',
               centre='"-50%"',
               top='"-100%"',
               '"0%"'),
        sep="")
}

alignmentBaseline <- function(vjust) {
  paste("alignment-baseline=",
        switch(vjust,
               baseline='"baseline"',
               bottom='"bottom"',
               center='"middle"',
               centre='"middle"',
               top='"top"',
               '"baseline"'),
        sep="")
}

# SVG transformations

svgtransform <- function(x, tlist) {
  result <- x
  if (length(tlist) > 0)
    for (i in length(tlist):1)
      result <- tlist[[i]](result)
  result
}

svgtx <- function(x, svgdev) {
  svgtransform(x, svgDevtx(svgdev))
}

svgty <- function(x, svgdev) {
  svgtransform(x, svgDevty(svgdev))
}

svgtw <- function(x, svgdev) {
  svgtransform(x, svgDevtw(svgdev))
}

svgth <- function(x, svgdev) {
  svgtransform(x, svgDevth(svgdev))
}

# Transform a "dimension" (not explicitly a width or height, e.g.,
# a circle radius)
svgtd <- function(d, svgdev) {
  pmin(svgtw(d, svgdev), svgth(d, svgdev))
}

# Assumes that (0, 0) is at bottom left
pushsvgTransform <- function(x, y, width, height, xscale, yscale,
                             svgdev) {
  xnum <- function(v) {
    if (xscale[1] < xscale[2]) 
      v - xscale[1]
    else 
      v - xscale[2]
  }
  xdenom <- function() {
    if (xscale[1] < xscale[2]) 
      xscale[2] - xscale[1]
    else 
      xscale[1] - xscale[2]
  }
  ynum <- function(v) {
    if (yscale[1] < yscale[2]) 
      v - yscale[1]
    else 
      v - yscale[2]
  }
  ydenom <- function() {
    if (yscale[1] < yscale[2]) 
      yscale[2] - yscale[1]
    else 
      yscale[1] - yscale[2]
  }
  pushsvgDevtx(function(v) { x + xnum(v)/xdenom()*width },
               svgdev)
  pushsvgDevty(function(v) { y + ynum(v)/ydenom()*height },
               svgdev)
  pushsvgDevtw(function(v) { v/xdenom()*width },
               svgdev)
  pushsvgDevth(function(v) { v/ydenom()*height },
               svgdev)
}

popsvgTransform <- function(svgdev) {
  popsvgDevtx(svgdev)
  popsvgDevty(svgdev)
  popsvgDevth(svgdev)
  popsvgDevtw(svgdev)
}

