library(grid)
library(gridSVG)

x11(width=6, height=6)

# Some default settings
push.viewport(viewport(gp=gpar(col="black", fill=NA)))

n <- 30
L <- 50 #mm
A <- 20 #mm
t <- seq(0, 2*pi, length=n)
x <- A*cos(t)
v <- -A*sin(t)
theta <- atan(x/L)
y <- L*cos(theta)

push.viewport(viewport(layout=grid.layout(4, 1,
                         widths=unit(2.2*A, "null"),
                         heights=unit(c(1, L + 10, 2.2*A, 2),
                           c("lines", "null", "null", "lines")),
                         respect=TRUE)))
push.viewport(viewport(layout.pos.row=2,
                       xscale=A * c(-1.1, 1.1),
                       yscale=c(0, L + 10)))
grid.rect()
grid.lines(x=grid.animate(unit(0.5, "npc"),
             unit(x, "native"),
             duration=5, rep=TRUE),
           y=grid.animate(unit(1, "npc"),
             unit(1, "npc") - unit(abs(y), "native"),
             duration=5, rep=TRUE))
grid.circle(x=grid.animate(unit(x, "native"),
              duration=5, rep=TRUE),
            y=grid.animate(unit(1, "npc") - unit(abs(y), "native"),
              duration=5, rep=TRUE),
            r=unit(1, "mm"),
            gp=gpar(fill="black"))
pop.viewport()

push.viewport(viewport(layout.pos.row=3,
                       xscale=A * c(-1.1, 1.1),
                       yscale=A * c(-1.1, 1.1)))
grid.rect()
grid.lines(x=unit(c(0, 0), "native"),
           gp=gpar(lty="dashed", col="grey"))
grid.lines(y=unit(c(0, 0), "native"),
           gp=gpar(lty="dashed", col="grey"))
grid.text("Displacement", y=unit(-1, "lines"))
grid.text("Velocity", x=unit(-1, "lines"), rot=90)
grid.circle(x=grid.animate(unit(x, "native"),
              duration=5, rep=TRUE),
            y=grid.animate(unit(v, "native"),
              duration=5, rep=TRUE),
            r=unit(1, "mm"),
            gp=gpar(fill="black"))
pop.viewport(2)

pop.viewport()

gridToSVG("pendulum.svg")

dev.off()
